@@ -33,7 +33,7 @@ my $has_cycle      = eval "use Devel::Cycle; 1;";
 use Devel::Peek;
 
 use vars qw($VERSION);
-$VERSION = '1.72';
+$VERSION = '1.74';
 
 use constant DEBUG => $ENV{PERLBAL_DEBUG} || 0;
 use constant DEBUG_OBJ => $ENV{PERLBAL_DEBUG_OBJ} || 0;
@@ -198,7 +198,7 @@ sub service {
 sub create_service {
     my $class = shift;
     my $name = shift;
-    
+
     unless (defined($name)) {
         $name = "____auto_".($service_autonumber++);
     }
@@ -263,6 +263,7 @@ sub run_manage_command {
 
     # expand variables
     $cmd =~ s/\$\{(.+?)\}/_expand_config_var($1)/eg;
+    $cmd =~ s/\$(\w+)/$ENV{$1}/g;
 
     $out ||= sub {};
     $ctx ||= Perlbal::CommandContext->new;
@@ -469,7 +470,7 @@ sub MANAGE_mime {
             return $mc->err("$arg1 not a defined extension.");
         }
     } else {
-        return $mc->err("Usage: list, remove <ext>, add <ext> <mime>");
+        return $mc->err("Usage: list, remove <ext>, set <ext> <mime>");
     }
 }
 
@@ -973,6 +974,45 @@ sub MANAGE_server {
     return $mc->err("unknown server option '$val'");
 }
 
+sub MANAGE_dumpconfig {
+    my $mc = shift;
+
+    while (my ($name, $pool) = each %pool) {
+        $mc->out("CREATE POOL $name");
+
+        if ($pool->can("dumpconfig")) {
+            foreach my $line ($pool->dumpconfig) {
+                $mc->out("  $line");
+            }
+        } else {
+            my $class = ref($pool);
+            $mc->out("  # Pool class '$class' is unable to dump config.");
+        }
+    } continue {
+        $mc->out("");
+    }
+
+    while (my ($name, $service) = each %service) {
+        $mc->out("CREATE SERVICE $name");
+
+        if ($service->can("dumpconfig")) {
+            foreach my $line ($service->dumpconfig) {
+                $mc->out("  $line");
+            }
+        } else {
+            my $class = ref($service);
+            $mc->out("  # Service class '$class' is unable to dump config.");
+        }
+
+        my $state = $service->{enabled} ? "ENABLE" : "DISABLE";
+        $mc->out("$state $name");
+    } continue {
+        $mc->out("");
+    }
+
+    return $mc->ok
+}
+
 sub MANAGE_reproxy_state {
     my $mc = shift;
     Perlbal::ReproxyManager::dump_state($mc->out);
@@ -1086,22 +1126,46 @@ sub MANAGE_load {
     my $last_case;
     my $last_class;
 
-    my $load = sub {
-        my $name = shift;
+    my $good_error;
+
+    # TODO case protection
+
+    foreach my $name ($fn, lc $fn, ucfirst lc $fn) {
         $last_case = $name;
         my $class = $last_class = "Perlbal::Plugin::$name";
-        my $rv = eval "use $class; $class->load; 1;";
-        return $mc->err($@) if ! $rv && $@ !~ /^Can\'t locate/;
-        return $rv;
-    };
+        my $file = $class . ".pm";
+        $file =~ s!::!/!g;
 
-    my $rv = $load->($fn) || $load->(lc $fn) || $load->(ucfirst lc $fn);
-    return $mc->err($@) unless $rv;
+        my $rv = eval "use $class; 1;";
 
-    $PluginCase{lc $fn} = $last_case;
-    $plugins{$last_case} = $last_class;
+        if ($rv) {
+            $good_error = undef;
+            last;
+        }
 
-    return $mc->ok;
+        # If we don't have a good error yet, start with this one.
+        $good_error = $@ unless defined $good_error;
+
+        # If the file existed perl will place an entry in %INC (though it will be undef due to compilation error)
+        if (exists $INC{$file}) {
+            $good_error = $@;
+            last;
+        }
+    }
+
+    unless (defined $good_error) {
+        my $rv = eval "$last_class->load; 1;";
+
+        if ($rv) {
+            $PluginCase{lc $fn} = $last_case;
+            $plugins{$last_case} = $last_class;
+            return $mc->ok;
+        }
+
+        $good_error = $@;
+    }
+
+    return $mc->err($good_error);
 }
 
 sub MANAGE_reload {
@@ -1173,7 +1237,6 @@ sub load_config {
     my $ctx = Perlbal::CommandContext->new;
     $ctx->verbose(0);
     while (my $line = <$fh>) {
-        $line =~ s/\$(\w+)/$ENV{$1}/g;
         return 0 unless run_manage_command($line, $writer, $ctx);
     }
     close($fh);
@@ -1295,8 +1358,13 @@ sub log {
     # simple logging functionality
     if ($foreground) {
         # syslog acts like printf so we have to use printf and append a \n
-        shift; # ignore the first parameter (info, warn, critical, etc)
-        printf(shift(@_) . "\n", @_);
+        shift; # ignore the first parameter (info, warn, crit, etc)
+        my $message = shift;
+        if (@_) {
+            printf("$message\n", @_);
+        } else {
+            print("$message\n");
+        }
     } else {
         # just pass the parameters to syslog
         Sys::Syslog::syslog(@_) if $Perlbal::syslog_open;
