@@ -183,8 +183,17 @@ sub start_reproxy_uri {
 sub try_next_uri {
     my Perlbal::ClientProxy $self = $_[0];
 
-    shift @{$self->{reproxy_uris}};
+    if ($self->{currently_reproxying}) {
+        # If we're currently reproxying to a backend, that means we want to try the next uri which is
+        # ->{reproxy_uris}->[0].
+    } else {
+        # Since we're not currently reproxying, that means we never got a backend in the first place,
+        # so we want to move on to the next uri which is ->{reproxy_uris}->[1] (shift one off)
+        shift @{$self->{reproxy_uris}};
+    }
+
     $self->{currently_reproxying} = undef;
+
     $self->start_reproxy_uri();
 }
 
@@ -251,10 +260,6 @@ sub backend_response_received {
     my Perlbal::ClientProxy $self = $_[0];
     my Perlbal::BackendHTTP $be = $_[1];
 
-    # a response means that we are no longer currently waiting on a reproxy, and
-    # don't want to retry this URI
-    $self->{currently_reproxying} = undef;
-
     # we fail if we got something that's NOT a 2xx code, OR, if we expected
     # a certain size and got back something different
     my $code = $be->{res_headers}->response_code + 0;
@@ -277,6 +282,11 @@ sub backend_response_received {
         $self->try_next_uri;
         return 1;
     }
+
+    # a response means that we are no longer currently waiting on a reproxy, and
+    # don't want to retry this URI
+    $self->{currently_reproxying} = undef;
+
     return 0;
 }
 
@@ -443,6 +453,13 @@ sub backend_finished {
     $self->close("backend_finished_while_unread_data");
 }
 
+# Called when this client is entering a persist_wait state, but before we are returned to base.
+sub persist_wait {
+    my Perlbal::ClientProxy $self = $_[0];
+    # We're in keepalive, and just completed a proxy request
+    $self->{service}->run_hooks('end_proxy_request', $self);
+}
+
 # called when we've sent a response to a user fully and we need to reset state
 sub http_response_sent {
     my Perlbal::ClientProxy $self = $_[0];
@@ -548,12 +565,13 @@ sub event_write {
     my Perlbal::ClientProxy $self = shift;
     print "ClientProxy::event_write\n" if Perlbal::DEBUG >= 3;
 
-    $self->SUPER::event_write;
-
     # obviously if we're writing the backend has processed our request
     # and we are responding/have responded to the user, so mark it so
     $self->{responded} = 1;
 
+    # will eventually, finally reset the whole object on completion
+    $self->SUPER::event_write;
+
     # trigger our backend to keep reading, if it's still connected
     if ($self->{backend_stalled} && (my $backend = $self->{backend})) {
         print "  unstalling backend\n" if Perlbal::DEBUG >= 3;
@@ -871,8 +889,9 @@ sub satisfy_request_from_cache {
 
         # If 'Last-Modified' is same as 'If-Modified-Since', send a 304
         if ($ims eq $lm) {
-            my $res_hd = Perlbal::HTTPHeaders->new_response(304);
+            my $res_hd = $self->{res_headers} = Perlbal::HTTPHeaders->new_response(304);
             $res_hd->header("Content-Length", "0");
+            $self->setup_keepalive($res_hd);
             $self->tcp_cork(1);
             $self->state('xfer_resp');
             $self->write($res_hd->to_string_ref);
@@ -1021,7 +1040,7 @@ sub send_buffered_upload {
 
     my $clen = $self->{req_headers}->content_length;
     if ($clen != $self->{buoutpos}) {
-        Perlbal::log('critical', "Content length of $clen declared but $self->{buoutpos} bytes written to disk");
+        Perlbal::log('crit', "Content length of $clen declared but $self->{buoutpos} bytes written to disk");
         return $self->_simple_response(500);
     }
 
@@ -1082,7 +1101,7 @@ sub buffered_upload_update {
 
             # throw errors back to the user
             if (! $self->{bufh}) {
-                Perlbal::log('critical', "Failure to open $fn for buffered upload output");
+                Perlbal::log('crit', "Failure to open $fn for buffered upload output");
                 return $self->_simple_response(500);
             }
 
@@ -1117,8 +1136,8 @@ sub buffered_upload_update {
         $self->{is_writing} = 0;
 
         # check for error
-        unless ($bytes) {
-            Perlbal::log('critical', "Error writing buffered upload: $!.  Tried to do $len bytes at $self->{buoutpos}.");
+        unless ($bytes > 0) {
+            Perlbal::log('crit', "Error writing buffered upload: $!.  Tried to do $len bytes at $self->{buoutpos}.");
             return $self->_simple_response(500);
         }
 
